@@ -26,33 +26,20 @@
 #define PANEL_DBUS_WRAPPER_PATH      PANEL_DBUS_PATH "/Wrapper/%d"
 #define PANEL_DBUS_WRAPPER_INTERFACE PANEL_DBUS_INTERFACE ".Wrapper"
 
-/* internal signals send over dbus */
-#define SIGNAL_SET_SIZE                 "a"
-#define SIGNAL_SET_ORIENTATION          "b"
-#define SIGNAL_SET_SCREEN_POSITION      "c"
-#define SIGNAL_SET_LOCKED               "d"
-#define SIGNAL_SAVE                     "e"
-#define SIGNAL_SHOW_CONFIGURE           "f"
-#define SIGNAL_SHOW_ABOUT               "g"
-#define SIGNAL_REMOVED                  "h"
-#define SIGNAL_WRAPPER_SET_SENSITIVE    "i"
-#define SIGNAL_WRAPPER_BACKGROUND_ALPHA "j"
-#define SIGNAL_WRAPPER_QUIT             "k"
-
 /* special types for dbus communication */
-#define PANEL_TYPE_DBUS_SET_MESSAGE \
+#define PANEL_TYPE_DBUS_SET_PROPERTY \
   dbus_g_type_get_struct ("GValueArray", \
-                          G_TYPE_STRING, \
+                          G_TYPE_UINT, \
                           G_TYPE_VALUE, \
                           G_TYPE_INVALID)
 
 #define PANEL_TYPE_DBUS_SET_SIGNAL \
   dbus_g_type_get_collection ("GPtrArray", \
-                              PANEL_TYPE_DBUS_SET_MESSAGE)
+                              PANEL_TYPE_DBUS_SET_PROPERTY)
 
 enum
 {
-  DBUS_SET_PROPERTY,
+  DBUS_SET_TYPE,
   DBUS_SET_VALUE
 };
 
