@@ -211,13 +211,26 @@ xfce_panel_plugin_provider_removed (XfcePanelPluginProvider *provider)
 gboolean
 xfce_panel_plugin_provider_remote_event (XfcePanelPluginProvider *provider,
                                          const gchar             *name,
-                                         const GValue            *value)
+                                         const GValue            *value,
+                                         guint                   *handle)
 {
+  const GValue *real_value = value;
+
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider), TRUE);
   panel_return_val_if_fail (name != NULL, TRUE);
   panel_return_val_if_fail (value == NULL || G_IS_VALUE (value), TRUE);
 
-  return (*XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->remote_event) (provider, name, value);
+  if (XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->remote_event != NULL)
+    {
+      if (real_value != NULL
+          && G_VALUE_HOLDS_UCHAR (real_value)
+          && g_value_get_uchar (real_value) == '\0')
+        real_value = NULL;
+
+      return (*XFCE_PANEL_PLUGIN_PROVIDER_GET_INTERFACE (provider)->remote_event) (provider, name, real_value, handle);
+    }
+
+  return FALSE;
 }
 
 
